/* THIS FILE, AccountsTables.SQL, SETS UP SOME SIMPLE ACCOUNTING TABLES */



/* CONNECT TO THE DATABASE BEFORE UPDATING IT */

CONNECT "c:\general\interbase\data\accounts.gdb"
USER "SYSDBA" PASSWORD "masterkey";


/* Table to facilitate multi-users */

CREATE TABLE owner
  	(
      	ref 	integer 		NOT NULL 	PRIMARY KEY,
      	owner	VARCHAR(50) 	NOT NULL,
	add1	VARCHAR(50)	DEFAULT NULL,
	add2	VARCHAR(50)	DEFAULT NULL,
	add3	VARCHAR(50)	DEFAULT NULL,
	add4	VARCHAR(50)	DEFAULT NULL,
	add5	VARCHAR(50)	DEFAULT NULL,
	add6	VARCHAR(50)	DEFAULT NULL
  	);




/* Table to faciliate multi-Brokers (in investment situation) */

CREATE TABLE broker
  	(
     	ref 	integer 		NOT NULL 	PRIMARY KEY,
     	broker	VARCHAR(50) 	NOT NULL,
	add1	VARCHAR(50)	DEFAULT NULL,
	add2	VARCHAR(50)	DEFAULT NULL,
	add3	VARCHAR(50)	DEFAULT NULL,
	add4	VARCHAR(50)	DEFAULT NULL,
	add5	VARCHAR(50)	DEFAULT NULL,
	add6	VARCHAR(50)	DEFAULT NULL
  	);



/* Table to record transaction types */

CREATE TABLE transType
	(
	ref		INTEGER	NOT NULL 	PRIMARY KEY,
	transType	VARCHAR(50)	NOT NULL
	);



/* Table to record the movement on assets */

CREATE TABLE assetTrans
  	(
      ref 		INTEGER		NOT NULL 	PRIMARY KEY,
	owner		INTEGER		NOT NULL,
	broker	INTEGER		NOT NULL,
	currency	INTEGER		NOT NULL,
	mRef		VARCHAR(50) 		DEFAULT NULL,
	assetRef	VARCHAR(50) 		DEFAULT NULL,
	transType	INTEGER		DEFAULT 1 	NOT NULL,
	transDate	DATE			NOT NULL,
	internalDate DATE			DEFAULT NULL,
	procDate	DATE			DEFAULT NULL,
	text		VARCHAR(50)		DEFAULT NULL,
	notes		BLOB			DEFAULT NULL,
	nlAsset	INTEGER		DEFAULT 0,
	nlBank	INTEGER		DEFAULT 0,
	nlReval	INTEGER		DEFAULT 0,
	num		DECIMAL(16,2)		DEFAULT 0.00,
	cost		DECIMAL(16,2)		DEFAULT 0.00,
	val		DECIMAL(16,2)		DEFAULT 0.00,
	proceeds	DECIMAL(16,2)		DEFAULT 0.00,
	prevDate	DATE			DEFAULT NULL,
	prevNum	DECIMAL(16,2)		DEFAULT 0.00,
	prevCost	DECIMAL(16,2)		DEFAULT 0.00,
	prevVal	DECIMAL(16,2)		DEFAULT 0.00
	);




/* Table to record the monetary movements (accounts) */

CREATE TABLE jnlTrans
	(
	ref		INTEGER		NOT NULL	PRIMARY KEY, 
	owner		INTEGER		NOT NULL,
	broker		INTEGER		NOT NULL,
	currency		VARCHAR(5)		DEFAULT "",
	transDate	DATE			NOT NULL,
	internalDate	DATE			DEFAULT NULL,
	procDate	DATE			DEFAULT NULL,
	assetRef		INTEGER		DEFAULT NULL,
	mRef		VARCHAR(50) 		DEFAULT NULL,
	description	VARCHAR(50) 		DEFAULT NULL,
	nlCodeRef	INTEGER		NOT NULL,
	cost		DECIMAL(16,2)		DEFAULT 0,
	val		DECIMAL(16,2)		DEFAULT 0
	);



/* Table to record the Nominal Ledger descriptions */

CREATE TABLE nlCode
(
ref		INTEGER	NOT NULL 	PRIMARY KEY,
nlCode		INTEGER	NOT NULL,
nlName		VARCHAR(50)	NOT NULL,
bsCat		VARCHAR(1)	NOT NULL,
reportCat	VARCHAR(2)	NOT NULL,

CHECK (bsCat IN ('b', 'p')),
CHECK (reportCat IN ('fa', 'ca','cl','ll','oe','re','cs','oh','tx'))
);


/* Table to hold the Nominal Ledger balance totals */

CREATE TABLE nl
(
nlCodeRef	INTEGER		NOT NULL,
owner		INTEGER		NOT NULL,
broker		INTEGER		NOT NULL,
currency		VARCHAR(5)		NOT NULL,
bal		DECIMAL(16,2)		DEFAULT 0,
val		DECIMAL(16,2)		DEFAULT 0,


PRIMARY KEY (owner, broker, currency, nlCodeRef)
);



/* Table to record the assets owned by the entity */

CREATE TABLE assets
	(
	ref		INTEGER 		NOT NULL 	PRIMARY KEY,
	assetRef		VARCHAR(50)		DEFAULT NULL,
	owner		INTEGER 		NOT NULL,
	broker		INTEGER		NOT NULL,
	currency		VARCHAR(5) 		DEFAULT '' 	NOT NULL,
	holding		VARCHAR(50) 		NOT NULL,
	assetNL		INTEGER 		DEFAULT 1,
	bankNL		INTEGER 		DEFAULT 1,
	revalNL		INTEGER 		DEFAULT 1,
	purchaseDate	DATE 			DEFAULT NULL,
	latestDate	DATE 			DEFAULT NULL,
	num 		DECIMAL(16,2)		DEFAULT 0	NOT NULL,
	cost		DECIMAL(10,2) 	DEFAULT 0 	NOT NULL,
	val		DECIMAL(10,2) 	DEFAULT 0 	NOT NULL,
	text		VARCHAR(50)		DEFAULT NULL,
	memo		BLOB(800,1)
	);



/* Table used to hold the next reference number for each table */

CREATE TABLE gen
(
assets_ref	INTEGER	DEFAULT 0,
nlCode_ref	INTEGER	DEFAULT 0,
jnlTrans_ref	INTEGER	DEFAULT 0,
assetTrans_ref	INTEGER	DEFAULT 0,
transType_ref	INTEGER	DEFAULT 0,
broker_ref	INTEGER	DEFAULT 0,
owner_ref	INTEGER	DEFAULT 0
);

generator end */



/* Commit above entries */

COMMIT;
