/* THIS IS A SQL FILE THAT ILLUSTRATES THE USE OF STORED PROCEDURES */



/* CONNECT TO THE DATABASE BEFORE YOU CAN UPDATE IT */

CONNECT "c:\Interbase\Data\IBDatabase.gdb"
USER "SYSDBA" PASSWORD "masterkey";




/* PROCEDURDS TO CREATE GENERATORS */
 

CREATE GENERATOR ADDRESS_GEN;
SET GENERATOR ADDRESS_GEN TO 30;


CREATE GENERATOR ORDER_GEN;
SET GENERATOR ORDER_GEN TO 138;


CREATE GENERATOR ORDERDETAIL_GEN;
SET GENERATOR ORDERDETAIL_GEN TO 168;


CREATE GENERATOR STOCK_GEN;
SET GENERATOR STOCK_GEN TO 0;





/* DEFINE THE TERMINATOR TO DISTINGUISH END OF STORED PROCEDURE */
/* FROM END OF SQL STATEMENT WITHIN STORED PROCEDURE            */


SET TERM ^ ;



/* PROCEDURES THAT CALL A GENERATOR AND RETURNS THE NEXT VALUE IN THE SEQUENCE      */
/* USED TO ALLOCATE A UNIQUE REFERENCE NUMBER TO A TABLE, SO THAT THE NEW REFERENCE */
/* NUMBER IS AVAILABLE IMMEDIATELY TO THE PROGRAM THAT IS DRIVING THE NEW RECORD    */


create procedure addressNewRef
RETURNS (ref integer)
AS
BEGIN
  ref = gen_id(address_gen,1);
END^

create procedure orderNewRef
RETURNS (ref integer)
AS
BEGIN
  ref = gen_id(order_gen,1);
END^

create procedure orderDetailNewRef
RETURNS (ref integer)
AS
BEGIN
  ref = gen_id(orderDetail_gen,1);
END^

create procedure stockNewRef
RETURNS (ref integer)
AS
BEGIN
  ref = gen_id(stock_gen,1);
END^



/* PROCEDURE THAT TAKES A REFERENCE NUMBER AS THE INPUT PARAMETER         */
/* THE OUTPUT IS DEFINED FOR USE IN DELPHI                                */
/* THE PROCEDURE SELECTS AND RETURNS DETAILS FOR THE REQUEST REFERENCE.   */
/* NOTE THE USE OF 'SUSPEND;' TO INSTRUCT TO INTERBASE THAT THE PROCEDURE */
/* HAS FINISHED WITH THE CURRENT RECORD, AND THE OUTPUT CAN BE WRITTEN    */
/* TO THE RESULT SET */ 

create PROCEDURE SelectedOrdersDetail(selectedRef integer)
RETURNS 
	(
	Ref			integer,
	OrderRef		integer,
	StockRef		integer,
	UnitsDelivered	integer,
	UnitsOrdered	integer,
	NumPerUnit		integer,
	QuantityOrdered	integer,
	QuantityDelivered	integer,
	PricePerUnit	decimal(14,2),
	TotalOrdered	decimal(14,2),
	TotalDelivered	decimal(14,2),
	OrderInputHTML	varchar(1500) 
	)
AS
BEGIN
	For	Select Ref,OrderRef,StockRef,UnitsDelivered,UnitsOrdered,
		NumPerUnit,QuantityOrdered,QuantityDelivered,
		PricePerUnit,TotalOrdered,TotalDelivered,OrderInputHTML
		FROM ordersDetail
		WHERE orderref=:selectedRef
		INTO	:Ref,:OrderRef,:StockRef,:UnitsDelivered,
		:UnitsOrdered,
		:NumPerUnit,:QuantityOrdered,:QuantityDelivered,
		:PricePerUnit,:TotalOrdered,:TotalDelivered,:OrderInputHTML
	do
	begin
		suspend;
	end
END^



/* PROCEDURE THAT TAKES A REFERENCE NUMBER AS THE INPUT PARAMETER */
/* THE OUTPUT IS DEFINED FOR USE IN DELPHI */
/* THE PROCEDURE SELECTS AND RETURNS DETAILS, THEN COMPUTES SOME HTML, */
/* WHICH IT ASSIGNS TO AN OUTPUT FIELD, BASED ON THE REQUESTED REFERENCE, */
/* PRIOR TO INCORPORATING IT WITHIN THE RESULT SET */

create procedure SelectedOrders(selectedRef integer)
RETURNS 
	(
	sRef			integer,
	sSTOCKCODE 		VARCHAR(20),
      sTYPE1 		VARCHAR(50),
	sTYPE2 		VARCHAR(50),
	sSTOCKSIZE 		VARCHAR(20),
	sSHORTDESCRIP 	VARCHAR(500),
	sLARGEPICTURE 	VARCHAR(500),
	sTHUMBNAIL 		VARCHAR(500),
	sSOLDPICTURE 	VARCHAR(500),
      sPRICEPERUNIT 	NUMERIC(15, 2),
	sQUANTITY 		INTEGER,
	sDISPLAY 		CHAR(1),
	sNUMPERUNIT 	INTEGER,
	sNUMUNITS 		integer,
	sLONGDESCRIP	VARCHAR(1500),
	sRefpointer		varchar(1200),
	sSelectQuantity	varchar(1200),
	sSelectArrow	varchar(1200),
	sSelectPicture	varchar(1200),

	oRef			integer,
	oOrderRef		integer,
	oStockRef		integer,
	oUnitsDelivered	integer,
	oUnitsOrdered	integer,
	oNumPerUnit		integer,
	oQuantityOrdered	integer,
	oQuantityDelivered	integer,
	oPricePerUnit	decimal(14,2),
	oTotalOrdered	decimal(14,2),
	oTotalDelivered	decimal(14,2),
	oORDERINPUTHTML 	varchar(1500)
	)
AS
	declare variable temp	integer;
BEGIN

	For	Select 
		s.Ref, s.stockcode, s.type1, s.type2,s.STOCKSIZE ,
		s.shortDescrip, s.LargePicture,
		s.Thumbnail, s.SoldPicture, s.PricePerUnit,s.Quantity,
		s.Display, s.NumPerUnit, s.NumUnits, s.LongDescrip,
		s.RefPointer,
		s.SelectQuantity, s.SelectArrow, s.SelectPicture,

		o.Ref,o.OrderRef,o.StockRef,o.UnitsDelivered,
		o.UnitsOrdered,o.NumPerUnit,o.QuantityOrdered,
		o.QuantityDelivered,o.PricePerUnit,o.TotalOrdered,
		o.TotalDelivered
		FROM stock s 
			left outer join
				selectedOrdersDetail(:selectedRef) o
			on s.Ref=o.StockRef
		INTO
		:sRef,:sstockcode, :stype1, :stype2, :sSTOCKSIZE,
		:sshortDescrip, :sLargePicture,
		:sThumbnail, :sSoldPicture, :sPricePerUnit,:sQuantity,
		:sDisplay, :sNumPerUnit, :sNumUnits, :sLongDescrip,
		:sRefPointer,
		:sSelectQuantity, :sSelectArrow, :sSelectPicture,
	
		:oRef,:oOrderRef,:oStockRef,:oUnitsDelivered,
		:oUnitsOrdered,:oNumPerUnit,:oQuantityOrdered,
		:oQuantityDelivered,:oPricePerUnit,:oTotalOrdered,
		:oTotalDelivered
	do
	begin
		temp=0;
		if (:oUnitsOrdered > 0) then
  			temp=oUnitsOrdered;

		oOrderInputHTML = 
                '<input type=text value="' || temp || '" name="' || cast(sRef as varchar(50)) || '" size="6">';

		suspend;

	end
END^


// SET THE TERMINATOR BACK TO THE CONVENTIONAL ';'

SET TERM ; ^



/* CREATE DOMAIN DEFINITIONS                                                     */
/* THIS SETS UP A BOOLEAN DATATYPE THAT DID NOT EXIST IN INTERBASE PRE VERSION 6 */


CREATE DOMAIN CHARBOOLEAN AS CHAR(1) default 'F' NOT NULL;



/* CREATE TABLES */
/* NOTE USE OF COMPUTED FIELD TO "CALCULATE" THE HTML */
/* FOR USE IN DELPHI HTML OUTPUT. */
/* THE FIRST EXAMPLE CREATES AN <a href> LINK. */
/* THE SECOND CREATES A LINK WHICH CALLS A JAVASCRIPT FUNCTION */
/* THE TABLES ARE THE BASIS OF A SYSTEM DESIGNED FOR WWW.CPKOI.COM */

CREATE TABLE ADDRESS (REF INTEGER NOT NULL,
        FIRSTNAME VARCHAR(50) default null,
        LASTNAME VARCHAR(50) default null,
        TITLE VARCHAR(10) default null,
        COMPANY VARCHAR(50) default null,
        ADD1 VARCHAR(80) default null,
        ADD2 VARCHAR(80) default null,
        ADD3 VARCHAR(80) default null,
        CITY VARCHAR(30) default null,
        COUNTY VARCHAR(30) default null,
        COUNTRY VARCHAR(30) default null,
        POSTCODE VARCHAR(20) default null,
        EMAIL VARCHAR(50) default null,
        FAX VARCHAR(30) default null,
        TEL VARCHAR(60) default null,
        MOBILE VARCHAR(30) default null,
        PSSWRD VARCHAR(50) default null,
        IPADDRESS VARCHAR(50) default null,
        DOB DATE default null,
        BROWSER VARCHAR(50) default null,
        OPSYS VARCHAR(50) default null,
        VISITS INTEGER default 0,
        ISACTIVE CHARBOOLEAN  default 'T',
        REFPOINTER COMPUTED BY 
          ('<a href="/bin/cpkoi.exe/AddressListing?ref=' || cast(ref as varchar(50)) ||
           '" target="displayWdw"><IMG SRC="/cpkoi/arrow.jpg"></A>' ),
PRIMARY KEY (REF));

CREATE TABLE LAYOUT (COL1SHORTDESCRIP VARCHAR(100),
        COL1LONGDESCRIP VARCHAR(500),
        COL1PICTURE VARCHAR(100),
        COL2SHORTDESCRIP VARCHAR(100),
        COL2LONGDESCRIP VARCHAR(500),
        COL2PICTURE VARCHAR(100),
        COL3PICTURE VARCHAR(100),
        COL3SHORTDESCRIP VARCHAR(100),
        COL3LONGDESCRIP VARCHAR(500),
        COL4LONGDESCRIP VARCHAR(500),
        COL4SHORTDESCRIP VARCHAR(100),
        COL4PICTURE VARCHAR(100),
        COL4THUMBNAIL VARCHAR(100),
        COL3THUMBNAIL VARCHAR(100),
        COL2THUMBNAIL VARCHAR(100),
        COL1THUMBNAIL VARCHAR(100),
        COL1 COMPUTED BY (
'<a href="javascript:description(''' || col1ShortDescrip || ''', ''' || col1LongDescrip || ''', ''' || col1Picture || ''')" ><IMG SRC="' || col1Thumbnail || '"></A>' 
),
        COL2 COMPUTED BY (
'<a href="javascript:description(''' || col2ShortDescrip || ''', ''' || col2LongDescrip || ''', ''' || col2Picture || ''')" ><IMG SRC="' || col2Thumbnail || '"></A>' 
),
        COL3 COMPUTED BY (
'<a href="javascript:description(''' || col3ShortDescrip || ''', ''' || col3LongDescrip || ''', ''' || col3Picture || ''')" ><IMG SRC="' || col3Thumbnail || '"></A>' 
),
        COL4 COMPUTED BY (
'<a href="javascript:description(''' || col4ShortDescrip || ''', ''' || col4LongDescrip || ''', ''' || col4Picture || ''')" ><IMG SRC="' || col4Thumbnail || '"></A>' 
));


CREATE TABLE ORDERS (REF INTEGER NOT NULL,
        CUSTOMERREF INTEGER NOT NULL,
        ORDERDATE DATE default null,
        CARDNUMBER VARCHAR(100) default null,
        EXPIRYDATE VARCHAR(20) default null,
        VALIDDATE VARCHAR(20) default null,
        CARDNAME VARCHAR(100) default null,
        AUTH VARCHAR(100) default null,
        AUTHDATE DATE default null,
        DATEDELIVERED DATE default null,
        ORDERTOTAL NUMERIC(15, 2) default 0,
        UNITSDELIVERED INTEGER default 0,
        UNITSORDERED INTEGER default 0,
        QUANTITYDELIVERED INTEGER default 0,
        QUANTITYORDERED INTEGER default 0,
        COMPLETED CHARBOOLEAN,
        LINKEDORDERREF INTEGER default null,
        BILLINGADDRESS VARCHAR(500) default null,
        DELIVERYADDRESS VARCHAR(500) default null,
        REFPOINTER COMPUTED BY (
'<a href="/bin/cpkoi.exe/OrdersDetailListing?ref=' ||
    cast(ref as varchar(50)) ||
    '" target="displayWdw"><IMG SRC="/cpkoi/arrow.jpg"></A>' ),
        POSTANDPACK NUMERIC(15, 2) default 10.00
,
        TOTALPLUSPACK COMPUTED BY (ordertotal + postandpack),
PRIMARY KEY (REF)
	);

CREATE TABLE ORDERSDETAIL (REF INTEGER NOT NULL,
        ORDERREF INTEGER NOT NULL,
        STOCKREF INTEGER NOT NULL,
        UNITSDELIVERED INTEGER default 0,
        UNITSORDERED INTEGER default 0,
        NUMPERUNIT INTEGER default 0,
        QUANTITYORDERED COMPUTED BY (UnitsOrdered * NumPerUnit),
        QUANTITYDELIVERED COMPUTED BY (UnitsDelivered * NumPerUnit),
        PRICEPERUNIT NUMERIC(15, 2),
        TOTALORDERED COMPUTED BY (PricePerUnit * UnitsOrdered),
        TOTALDELIVERED COMPUTED BY (PricePerUnit * UnitsDelivered),
        ORDERINPUTHTML COMPUTED BY (
'<input type=text name="' || 
     cast(stockref as varchar(50)) ||    '" value="' ||
     unitsordered || '">'
),
PRIMARY KEY (REF));

CREATE TABLE STOCK (REF INTEGER NOT NULL,
        STOCKCODE VARCHAR(20) default null,
        TYPE1 VARCHAR(50) default null,
        TYPE2 VARCHAR(50) default null,
        STOCKSIZE VARCHAR(20) default null,
        SHORTDESCRIP VARCHAR(100) default null,
        LARGEPICTURE VARCHAR(100) default null,
        THUMBNAIL VARCHAR(100) default null,
        SOLDPICTURE VARCHAR(100) default null,
        PRICEPERUNIT NUMERIC(15, 2) default 0,
        QUANTITY INTEGER default 0,
        DISPLAY CHARBOOLEAN  default 'T',
        NUMPERUNIT INTEGER default 1,
        NUMUNITS COMPUTED BY (div(quantity, numPerUnit)),
        LONGDESCRIP VARCHAR(500) default null,
        SELECTQUANTITY COMPUTED BY (
'<Input Type=text value="0" size="6" name=' || cast(ref as varchar(50)) || '>'
),
        SELECTARROW COMPUTED BY (
'<a href="javascript:description(''' || cast(ref as varchar(50)) || ''',''' || ShortDescrip || ''',''' || LongDescrip || ''',''' || LargePicture || ''')" ><IMG SRC="/cpKoi/arrow.jpg"></A>' 
),
        SELECTPICTURE COMPUTED BY (
'<a href="javascript:description(''' || cast(ref as varchar(50)) ||''', ''' || ShortDescrip || ''', ''' || LongDescrip || ''', ''' || LargePicture || ''')" ><IMG SRC="' || Thumbnail || '"></A>' 
),
        REFPOINTER COMPUTED BY (
'<a href="/cpkoi/cpKoi.exe/StockListing?ref=' ||
    cast(ref as varchar(50)) ||
    '" target="displayWdw"><IMG SRC="/cpKoi/arrow.jpg"></A>' ),
PRIMARY KEY (REF));




/* PROCEDURE THAT INSERTS TEST DATA INTO THE TABLE ADDRESS */


INSERT INTO ADDRESS
(firstName, lastName, COMPANY, EMAIL)
VALUES
( 'Sue', 'Topchik', 'Software Design UK', 'sue@softwaredesign.co.uk')


