/* THIS FILE, QUERIES.SQL, PROVIDES EXAMPLES OF SQL QUERIES



/* THIS PROCEDURE PRODUCES A RESULT SET FOR USE BY OTHER PROCEDURES */

CREATE PROCEDURE gainsInPeriod
  (startDate Date, endDate Date, nlcoderef integer)
RETURNS
  (assetRef integer)
as
begin
  for select distinct(assetRef) from jnlTrans
  where cost<>0 
  and transdate >= :startDate and transdate <= :endDate
  and nlcoderef=:nlcoderef
  order by assetRef
  into :assetRef
  
  do

  begin

    suspend;

  end

end^



/* THIS PROCEDURE JOINS A NUMBER OF TABLES TOGETHER FOR REPORTING PURPOSES */
/* NOTE THAT IT USES THE ABOVE STORED PROCEDURE (gainsInPeriod), AS THE 'TABLE' */
/* WHICH HAS 3 INPUT PARAMETERS - START DATE, END DATE AND NOMINAL CODE    */

/* Establish the "cost and number" movement on assets  */
/* for which gains earned during year (nlcoderef=21)   */

select a.owner, a.assetRef,s.holding, a.transdate,t.transType, a.num, a.cost, a.broker
from
assetTrans a inner join gainsInPeriod('3/31/1998','4/1/1999',21) g
   on a.assetRef = g.assetRef
left outer join assets s
  on  s.ref=a.assetRef 
left outer join transtype t
  on a.transType = t.ref
where a.cost <> 0 and a.currency='$'
order by owner, broker, a.assetref, transdate



/* ANOTHER EXAMPLE OF THE SAME TYPE OF PROCEDURE                   */
/* List the accounting entries for all accounting transactions     */
/* for all owners where there were gains during the tax year,      */
/* excluding gains b/f (t.transType=2 and t.transdate<'4/1/1998')  */

select j.owner, j.assetRef, s.holding, j.transdate, j.description, j.currency, n.nlname , 
  t.transType, j.cost
from 
jnlTrans j inner join gainsInPeriod('3/31/1998','4/1/1999',21) g
  on j.assetRef = g.assetRef
left outer join assetTrans t
  on j.assetRef=t.assetRef and j.transdate=t.transDate
left outer join nlcode n
  on n.ref = j.nlcoderef
left outer join assets s
  on s.ref=j.assetRef
where currency='' and j.cost <> 0 and j.assetRef <> 0 
  and not (t.transType=2 and t.transdate<'4/1/1998')
order by j.owner, j.broker, j.assetRef, j.transdate



/* THIS PROCEDURE TAKES AN INPUT, RETURNS AN OUTPUT SET */
/* AND ALSO UPDATES A TABLE CALCLUATED INFORMATION      */


CREATE PROCEDURE assetTransPost (latestDate DATE )
RETURNS (totNum NUMERIC(16,2), totCost NUMERIC(16,2), totVal NUMERIC(16,2))
AS
  DECLARE VARIABLE jNum DECIMAL(16, 2);
  DECLARE VARIABLE jCost DECIMAL(16, 2);
  DECLARE VARIABLE jVal DECIMAL(16, 2);
  DECLARE VARIABLE jAssetRef INTEGER;
  DECLARE VARIABLE aRef INTEGER;
  DECLARE VARIABLE jProcDate DATE;

BEGIN
  totNum = 0;
  totCost = 0;
  totVal = 0;

  FOR SELECT j.num, j.cost, j.Val, j.AssetRef, a.Ref
  FROM assetTrans j, assets a
  WHERE j.assetRef = a.Ref and j.procDate < "1/1/1990"
  INTO :jNum, :jCost, :jVal, :jAssetRef, :aRef
  DO
  BEGIN

    /* First process in Stored Procedure */
    UPDATE assets SET
        num = num + :jNum,Cost =cost + :jCost, Val  = val  + :jVal,
        latestDate = :latestDate
    WHERE ref = :jAssetRef;

    /* Next process in Stored Procedure */
    /* Get cumulative totals for entire dataset */

    totNum = totNum + :jNum;
    totCost = totCost + :jCost;
    totVal = totVal + :jVal;


  END

  /* Next update, after completion of the For..Select statement */

  UPDATE assetTrans SET
  	procDate = :latestDate
  WHERE procDate < "1/1/1990";


  /* GO TO NEXT RECORD */

  suspend;


END^



/* THIS PROCEDURE PRODUCES A REPORT WHICH 'CONVERTS' A      */
/* +VE NUMBER AS A DEBIT AND A -VE NUMBER AS A CREDIT       */
/* NOTE THAT THE PROCEDURE USES TWO "FOR SELECT" STATEMENTS */

CREATE PROCEDURE transInfo
RETURNS (latestRef INTEGER, cost DECIMAL(15,2), val DECIMAL(15,2), 
crCost DECIMAL(15,2), crVal DECIMAL(15,2) )
AS
BEGIN
FOR SELECT max(ref) 
FROM transactions
INTO :latestRef

DO	suspend;

FOR SELECT sum(cost), sum(val)
FROM transactions
WHERE nlDebit > 0 AND nlUpdated is null
INTO :drCost, :drVal

DO	suspend;

FOR SELECT sum(cost), sum(val)
FROM transactions
WHERE nlCredit > 0 AND nlUpdated is null
INTO :crCost, :crVal

DO
	Suspend;

END^



/* THIS PROCEDURE GROUPS ITEMS INTO A COMPLEX GROUPING */
/* FOR REPORTING PURPOSES                              */

CREATE PROCEDURE axOwnerBrokerSQL (currencySel VARCHAR(5))
RETURNS (nlCode integer, nlName varchar(50), bal decimal(16,2), val decimal(16,2), bsCat varchar(2), reportCat varchar(2), currency varChar(5), counted integer, balTot decimal(16,2), valTot decimal(16,2))
AS BEGIN

select count(nlCode), sum(bal), sum(val) 
FROM nl n, nlCode c
WHERE n.nlCodeRef = c.ref and 
               currency = :currencySel and (val <> 0 or bal <> 0)
INTO :counted, :balTot, :valTot;


FOR SELECT c.nlcode, c.nlName, c.bsCat, c.reportCat, 
               n.currency, sum(bal), sum(val)
FROM nl n, nlCode c
WHERE n.nlCodeRef = c.ref and 
               currency = :currencySel and (val <> 0 or bal <> 0)

GROUP BY  c.nlcode, c.nlName, c.bsCat, c.reportCat, n.currency
ORDER BY c.nlcode, c.nlName, c.bsCat, c.reportCat, n.currency

INTO :nlCode, :nlName, :bsCat, :reportCat, :currency, :bal, :val



DO 
BEGIN
   SUSPEND;
END


END^



/* ILLUSTRATION OF USE OF A TRIGGER */

CREATE TRIGGER set_emp_no FOR employee
BEFORE INSERT
AS
BEGIN
  new.ref = gen_id(employee_gen,1);
END^

SET TERM ;^
